@@ -374,12 +374,6 @@ FeatureModelGraph::setUseNVGL(bool value)
     }
 }
 
-void
-FeatureModelGraph::setFeatureQueryBufferWidthAsPercentage(double value)
-{
-    _featureQueryBufferWidthAsPercentage = value;
-}
-
 void
 FeatureModelGraph::setSession(Session* value)
 {
@@ -1330,30 +1324,6 @@ FeatureModelGraph::buildTile(
         if (key)
             query.tileKey() = *key;
 
-        // buffer the query?
-        if (_featureQueryBufferWidthAsPercentage.isSet())
-        {
-            if (query.bounds().isSet())
-            {
-                auto* fp = featureSource->getFeatureProfile();
-                double w = width(query.bounds().value());
-                double h = height(query.bounds().value());
-                double buffer = sqrt(w * h) * _featureQueryBufferWidthAsPercentage.value();
-                query.buffer() = Distance(buffer, fp->getSRS()->getUnits());
-            }
-            else if (query.tileKey().isSet())
-            {
-                double w = query.tileKey()->getExtent().width();
-                double h = query.tileKey()->getExtent().height();
-                double buffer = sqrt(w * h) * _featureQueryBufferWidthAsPercentage.value();
-                query.buffer() = Distance(buffer, query.tileKey()->getProfile()->getSRS()->getUnits());
-            }
-            else
-            {
-                OE_WARN << LC << "Requested a buffer width as a percentage but no bounds or tilekey was set" << std::endl;
-            }
-        }
-
         // does the level have a style name set?
         if (level.styleName().isSet())
         {
@@ -1820,29 +1790,26 @@ FeatureModelGraph::createStyleGroup(const Style&          style,
 
     FilterContext context(contextPrototype);
 
-    if (_options.autoCropFeatures() == true)
+    // First Crop the feature set to the working extent.
+    // Note: There is an obscure edge case that can happen is a feature's centroid
+    // falls exactly on the crop extent boundary. In that case the feature can
+    // show up in more than one tile. It's rare and not trivial to mitigate so for now
+    // we have decided to do nothing. :)
+    CropFilter crop(
+        _options.layout().isSet() && _options.layout()->cropFeatures() == true ?
+        CropFilter::METHOD_CROP_TO_EXTENT : CropFilter::METHOD_CENTROID);
+
+    context = crop.push(workingSet, context);
+
+    // next, if the usable extent is less than the full extent (i.e. we had to clamp the feature
+    // extent to fit on the map), calculate the extent of the features in this tile and
+    // crop to the map extent if necessary. (Note, if cropFeatures was set to true, this is
+    // already done)
+    if (_featureExtentClamped && _options.layout().isSet() && _options.layout()->cropFeatures() == false)
     {
-        // First Crop the feature set to the working extent.
-        // Note: There is an obscure edge case that can happen is a feature's centroid
-        // falls exactly on the crop extent boundary. In that case the feature can
-        // show up in more than one tile. It's rare and not trivial to mitigate so for now
-        // we have decided to do nothing. :)
-        CropFilter crop(
-            _options.layout().isSet() && _options.layout()->cropFeatures() == true ?
-            CropFilter::METHOD_CROP_TO_EXTENT : CropFilter::METHOD_CENTROID);
-
-        context = crop.push(workingSet, context);
-
-        // next, if the usable extent is less than the full extent (i.e. we had to clamp the feature
-        // extent to fit on the map), calculate the extent of the features in this tile and
-        // crop to the map extent if necessary. (Note, if cropFeatures was set to true, this is
-        // already done)
-        if (_featureExtentClamped && _options.layout().isSet() && _options.layout()->cropFeatures() == false)
-        {
-            context.extent() = _usableFeatureExtent;
-            CropFilter crop2(CropFilter::METHOD_CROP_TO_EXTENT);
-            context = crop2.push(workingSet, context);
-        }
+        context.extent() = _usableFeatureExtent;
+        CropFilter crop2(CropFilter::METHOD_CROP_TO_EXTENT);
+        context = crop2.push(workingSet, context);
     }
 
     // finally, compile the features into a node.
@@ -1886,9 +1853,6 @@ FeatureModelGraph::createStyleGroup(const Style&          style,
 
     FilterContext context(_session.get(), featureProfile, GeoExtent(featureProfile->getSRS(), cellBounds), index);
 
-    //if (!query.tileKey()->is(14, 2847, 6583))
-    //    return {};
-
     // query the feature source:
     osg::ref_ptr<FeatureCursor> cursor = _session->getFeatureSource()->createFeatureCursor(
         query,
