@@ -178,10 +178,9 @@ GeodeticGraticule::init()
     // Read the resolutions from the config.
     if (options().resolutions().isSet())
     {
-        auto tokens = StringTokenizer()
-            .delim(" ")
-            .tokenize(options().resolutions().value());
-
+        StringTokenizer tok(" ");
+        StringVector tokens;
+        tok.tokenize(*options().resolutions(), tokens);
         for (unsigned int i = 0; i < tokens.size(); i++)
         {
             double r = as<double>(tokens[i], -1.0);
@@ -519,7 +518,6 @@ GeodeticGraticule::getViewExtent(osgUtil::CullVisitor* cullVisitor) const
     // side of the globe
     osg::Vec3d eye = osg::Vec3d(0,0,0) * invmv;
 
-    // TODO: fix this, hard coded to earth
     const osgEarth::SpatialReference* srs = osgEarth::SpatialReference::create("epsg:4326");
 
     double nearPlane, farPlane;
@@ -596,7 +594,6 @@ GeodeticGraticule::getViewExtent(osgUtil::CullVisitor* cullVisitor) const
     osgEarth::GeoPoint center;
     center.fromWorld(srs, bs.center());
 
-    // TODO: fix this, it's hard-coded to earth
     double radiusDegrees = bs.radius() / 111000.0;
 
     // Try to clamp the maximum radius so far out views don't go wacky.
@@ -622,7 +619,6 @@ GeodeticGraticule::updateLabels()
         return;
     }
 
-    //TODO: fix earth hard coding
     const osgEarth::SpatialReference* srs = osgEarth::SpatialReference::create("wgs84");
 
     std::lock_guard<std::mutex> lock(_cameraDataMapMutex);
