@@ -129,11 +129,10 @@ PolygonizeLinesOperator::PolygonizeLinesOperator(
 
 osg::Geometry*
 PolygonizeLinesOperator::operator()(
-    osg::Vec3Array* verts, 
-    osg::Vec3Array* normals,
-    float width,
-    Callback* callback,
-    bool twosided) const
+    osg::Vec3Array*  verts, 
+    osg::Vec3Array*  normals,
+    Callback*        callback,
+    bool             twosided) const
 {
     // number of verts on the original line.
     unsigned lineSize = verts->size();
@@ -144,6 +143,7 @@ PolygonizeLinesOperator::operator()(
 
     const Stroke& stroke = _line->stroke().get();
 
+    float width            = Distance(stroke.width().get(), stroke.widthUnits().get()).as(Units::METERS);
     float halfWidth        = 0.5f * width;
     float maxRoundingAngle = asin( stroke.roundingRatio().get() );
     float minPixelSize     = stroke.minPixels().getOrUse( 0.0f );
@@ -642,13 +642,6 @@ PolygonizeLinesFilter::push(FeatureList& input, FilterContext& cx)
     {
         Feature* f = i->get();
 
-        Distance lineWidth(1, Units::METERS);
-        if (line && line->stroke().isSet() && line->stroke()->width().isSet())
-        {
-            lineWidth = line->stroke()->width()->eval(f, cx);
-            //lineWidth = f->eval(line->stroke()->width().value(), &cx);
-        }
-
         // iterate over all the feature's geometry parts. We will treat
         // them as lines strings.
         GeometryIterator parts( f->getGeometry(), false );
@@ -667,7 +660,7 @@ PolygonizeLinesFilter::push(FeatureList& input, FilterContext& cx)
             transformAndLocalize( part->asVector(), featureSRS, verts, normals, mapSRS, _world2local, makeECEF );
 
             // turn the lines into polygons.
-            osg::Geometry* geom = polygonize(verts, normals, lineWidth.as(Units::METERS));
+            osg::Geometry* geom = polygonize( verts, normals );
 
             // install.
             geode->addDrawable( geom );
